/*!\file Misfitx
 * \brief: compute misfit between observations and model
 */

#include "./Misfitx.h"

#include "../shared/shared.h"
#include "../include/include.h"
#include "../toolkits/toolkits.h"
#include "../EnumDefinitions/EnumDefinitions.h"
#include "../SurfaceAreax/SurfaceAreax.h"

void Misfitx( double* pJ, DataSet* elements,DataSet* nodes, DataSet* vertices, DataSet* loads, DataSet* materials,Parameters* parameters,
			int analysis_type,int sub_analysis_type){
	
	/*Intermediary*/
	int fit;
	double S;

	/*output: */
	double J;
	double J_sum;
	
	/*First, get elements and loads configured: */
	elements->Configure(elements,loads, nodes,vertices, materials,parameters);
	nodes->Configure(elements,loads, nodes,vertices, materials,parameters);
	parameters->Configure(elements,loads, nodes,vertices, materials,parameters);

	/*Compute surface area: */
	SurfaceAreax(&S,elements,nodes,vertices, loads,materials,parameters,analysis_type,sub_analysis_type);

	/*add surface area to elements :*/
	elements->UpdateInputsFromVector(&S,SurfaceAreaEnum,ConstantEnum);
	

	/*Compute Misfit: */
	elements->Misfit(&J,analysis_type,sub_analysis_type);

	/*Sum all J from all cpus of the cluster:*/
	MPI_Reduce (&J,&J_sum,1,MPI_DOUBLE,MPI_SUM,0,MPI_COMM_WORLD );
	MPI_Bcast(&J_sum,1,MPI_DOUBLE,0,MPI_COMM_WORLD); 
	J=J_sum;

	/*Assign output pointers: */
	*pJ=J;
}
